@@ -5,93 +5,14 @@ use Log::Any '$log';
 use Moo;
 
 extends 'Perinci::To::PackageBase';
+with    'Perinci::To::Text::AddDocLinesRole';
 
-has wrap => (is => 'rw', default=>sub{1});
-
-our $VERSION = '0.09'; # VERSION
+our $VERSION = '0.10'; # VERSION
 
 sub BUILD {
     my ($self, $args) = @_;
 }
 
-sub add_doc_lines {
-    my $self = shift;
-    my $opts;
-    if (ref($_[0]) eq 'HASH') { $opts = shift }
-    $opts //= {};
-
-    my @lines = map { $_ . (/\n\z/s ? "" : "\n") }
-        map {/\n/ ? split /\n/ : $_} @_;
-
-    my $indent = $self->indent x $self->indent_level;
-    my $wrap = $opts->{wrap} // $self->wrap;
-    if ($wrap) {
-        require Text::Wrap;
-
-        # split into paragraphs, merge each paragraph text into a single line
-        # first
-        my @para;
-        my $i = 0;
-        my ($start, $type);
-        $type = '';
-        #$log->warnf("lines=%s", \@lines);
-        for (@lines) {
-            if (/^\s*$/) {
-                if (defined($start) && $type ne 'blank') {
-                    push @para, [$type, [@lines[$start..$i-1]]];
-                    undef $start;
-                }
-                $start //= $i;
-                $type = 'blank';
-            } elsif (/^\s{4,}\S+/ && (!$i || $type eq 'verbatim' ||
-                         (@para && $para[-1][0] eq 'blank'))) {
-                if (defined($start) && $type ne 'verbatim') {
-                    push @para, [$type, [@lines[$start..$i-1]]];
-                    undef $start;
-                }
-                $start //= $i;
-                $type = 'verbatim';
-            } else {
-                if (defined($start) && $type ne 'normal') {
-                    push @para, [$type, [@lines[$start..$i-1]]];
-                    undef $start;
-                }
-                $start //= $i;
-                $type = 'normal';
-            }
-            #$log->warnf("i=%d, lines=%s, start=%s, type=%s",
-            #            $i, $_, $start, $type);
-            $i++;
-        }
-        if (@para && $para[-1][0] eq $type) {
-            push @{ $para[-1][1] }, [$type, [@lines[$start..$i-1]]];
-        } else {
-            push @para, [$type, [@lines[$start..$i-1]]];
-        }
-        #$log->warnf("para=%s", \@para);
-
-        my $columns //= 80 - length($indent);
-        for my $para (@para) {
-            if ($para->[0] eq 'blank') {
-                push @{$self->doc_lines}, @{$para->[1]};
-            } else {
-                if ($para->[0] eq 'normal') {
-                    for (@{$para->[1]}) {
-                        s/\n/ /g;
-                    }
-                    $para->[1] = [join("", @{$para->[1]}) . "\n"];
-                }
-                #$log->warnf("para=%s", $para);
-                push @{$self->doc_lines},
-                    Text::Wrap::wrap($indent, $indent, @{$para->[1]});
-            }
-        }
-    } else {
-        push @{$self->doc_lines},
-            map {"$indent$_"} @lines;
-    }
-}
-
 sub doc_gen_summary {
     my ($self) = @_;
 
@@ -224,7 +145,7 @@ Perinci::To::Text - Generate text documentation from Rinci package metadata
 
 =head1 VERSION
 
-version 0.09
+version 0.10
 
 =head1 AUTHOR
 
